/*
 * Merlin++: C++ Class Library for Charged Particle Accelerator Simulations
 * Copyright (c) 2001-2018 The Merlin++ developers
 * This file is covered by the terms the GNU GPL version 2, or (at your option) any later version, see the file COPYING
 * This file is derived from software bearing the copyright notice in merlin4_copyright.txt
 */

#include "PhysicalUnits.h"
#include "NumericalConstants.h"

namespace PhysicalUnits
{

//Area
const double barn = 1.0e-28;    // m^2

// length
const double meter = 1.0;
const double centimeter = 1.0e-02 * meter;
const double millimeter = 1.0e-03 * meter;
const double micrometer = 1.0e-06 * meter;
const double nanometer  = 1.0e-09 * meter;

// time
const double second = 1;
const double millisecond = 1.0e-03 * second;
const double microsecond = 1.0e-06 * second;
const double nanosecond = 1.0e-09 * second;
const double picosecond = 1.0e-12 * second;
const double minute = 60 * second;
const double hour = 60 * minute;
const double day = 24 * hour;
const double year = 365 * day;

// energy
const double eV = 1.0e-9;
const double keV = 1.0e+3 * eV;
const double MeV = 1.0e+6 * eV;
const double GeV = 1.0e+9 * eV;
const double TeV = 1.0e+12 * eV;

// voltage
const double Volt = 1.0e-9;
const double kV = 1.0e+3 * Volt;
const double MV = 1.0e+6 * Volt;
const double GV = 1.0e+9 * Volt;
const double TV = 1.0e+12 * Volt;

// Frequency
const double Hz = 1.0;
const double kHz = 1.0e+3 * Hz;
const double MHz = 1.0e+6 * Hz;
const double GHz = 1.0e+9 * Hz;
const double THz = 1.0e+12 * Hz;

// Angle
const double radian = 1.0;
const double milliradian = 1.0e-3 * radian;
const double microradian = 1.0e-6 * radian;
const double degree = 0.0174532925199 * radian;

// magnetic field
const double Tesla = 1.0;
const double kGauss = 1.0e-1 * Tesla;
const double Gauss = 1.0e-3 * kGauss;

// mass
const double amu = 931.494028 * MeV; // atomic mass unit in MeV

}
